/* ********************************************************

   chromatic polynomial of G (Algorithm 1)

   LEDA 5.0
   g++ -I$LEDAROOT/incl -L$LEDAROOT algorithm_1.cpp -lW -lP -lG -lL -lX11 -lm

   ******************************************************** */


#include <LEDA/graph/graph.h>
#include <LEDA/core/array.h>
#include <iostream>
using namespace leda;
using namespace std;

int complete_graph(graph G);
int algorithm_1(graph &G,int col);
void edge_add(graph &G);
void edge_contraction(graph &G);

int main()
{
  graph G;
  G.read("graph.gw");

  int color;
  cin >> color;

  cout << algorithm_1(G,color) << endl;

}

int algorithm_1(graph &G,int col)
{
  int nu = G.number_of_nodes();
  int i = nu,j = 1;

  if(complete_graph(G)){
    for(;i>0;i--)
      j=j*col;
    return j;
  }
  else {
    graph G_1 = G;
    node v;
    edge_add(G_1);
    graph G_2 = G_1;
    edge_contraction(G_2);
    int p_1 = algorithm_1(G_1,col);
    int p_2 = algorithm_1(G_2,col);
    return p_1+p_2;
  }
}

int complete_graph(graph G)
{
  node v;
  int i=G.number_of_nodes();
  int j;

  forall_nodes(v,G){
    j=G.degree(v);
    if(i!=j+1)
      return 0;
  }

  return 1;
}

void edge_add(graph &G)
{
  G.make_undirected();
  node v,w;

  node_array<int> node_ord(G,0);

  int v_num = G.number_of_nodes();

  for(;;){
    v = G.choose_node();
    if(G.degree(v) < v_num-1)
      break;
  }
  node_ord[v]=1;
  forall_adj_nodes(w,v)
    node_ord[w]=1;

  forall_nodes(w,G)
    if(node_ord[w] == 0)
      break;

  G.new_edge(v,w);
}

void edge_contraction(graph &G)
{

  node new_v = G.new_node();
  G.make_undirected();

  edge e = G.last_edge();

  node  v = G.source(e);
  node  w = G.opposite(v,e);


  node_array<int> node_ord(G,0);
  node x;

  forall_adj_nodes(x,v)
    node_ord[x]=1;
  forall_adj_nodes(x,w)
    node_ord[x]=1;
  node_ord[v]=0;
  node_ord[w]=0;

  forall_nodes(x,G)
    if(node_ord[x] == 1)
      G.new_edge(new_v,x);

  G.del_node(v);
  G.del_node(w);

}
